import json
import logging
import os

import gemmi
import matplotlib.pyplot as plt
import pandas as pd
import plotly.express as px

logger = logging.getLogger("relion.cryolo_relion_it.icebreaker_histogram")


def create_json_histogram(working_directory):
    icebreaker_path = working_directory / "External" / "Icebreaker_group"
    if icebreaker_path.joinpath("particles.star").is_file():
        data = extract_ice_column(icebreaker_path / "particles.star")
        if data is None:
            logger.warning("No ice thickness data extracted.")
            return None
    else:
        return None
    df = pd.DataFrame(data, columns=["Relative estimated ice thickness"])
    fig = px.histogram(
        df,
        x="Relative estimated ice thickness",
        title="Histogram of Icebreaker estimated ice thickness <br>Total number of particles = "
        + str(len(data)),
    )
    full_json_path_object = icebreaker_path / "ice_hist.json"
    fig.write_json(
        os.fspath(full_json_path_object)
    )  # This plotly version doesn't support Path objects with write_json()
    return full_json_path_object


def create_pdf_histogram(working_directory):
    icebreaker_path = working_directory / "External" / "Icebreaker_group"
    if icebreaker_path.joinpath("particles.star").is_file():
        data = extract_ice_column(icebreaker_path / "particles.star")
    else:
        return None
    plt.hist(x=data, bins="auto", rwidth=0.9)
    plt.xlabel("Relative estimated ice thickness")
    plt.ylabel("Number of particles")
    plt.title("Histogram of Icebreaker estimated ice thickness")
    plt.legend(["Total number of particles = " + str(len(data))])
    full_pdf_path_object = icebreaker_path / "ice_hist.pdf"
    plt.savefig(
        os.fspath(full_pdf_path_object)
    )  # This matplotlib version doesn't support Path objects with savefig()
    return full_pdf_path_object


def extract_ice_column(icebreaker_star_file_path):
    particles = gemmi.cif.read_file(os.fspath(icebreaker_star_file_path))
    try:
        data_as_dict = json.loads(particles.as_json())["particles"]
        ice_group = data_as_dict["_rlnhelicaltubeid"]
        return ice_group
    except KeyError:
        return []



from sqlalchemy import Column
from sqlalchemy.dialects.mysql import INTEGER
from sqlalchemy.ext.declarative import declarative_base

Base = declarative_base()


class ZocaloBuffer(Base):
    __tablename__ = "ZocaloBuffer"

    AutoProcProgramID = Column(
        INTEGER(10),
        primary_key=True,
        comment="Reference to an existing AutoProcProgram",
        autoincrement=False,
        nullable=False,
    )
    UUID = Column(
        INTEGER(10),
        primary_key=True,
        comment="AutoProcProgram-specific unique identifier",
        autoincrement=False,
        nullable=False,
    )
    Reference = Column(
        INTEGER(10),
        comment="Context-dependent reference to primary key IDs in other ISPyB tables",
    )


def buffer_url() -> str:
    import ispyb.sqlalchemy

    sqlalchemy_url = ispyb.sqlalchemy.url()
    local_url = "/".join(sqlalchemy_url.split("/")[:-1]) + "/zocalo"
    return local_url



import logging
from collections import namedtuple

from relion._parser.jobtype import JobType

logger = logging.getLogger("relion._parser.autopick")

ParticlePickerInfo = namedtuple(
    "ParticlePickerInfo",
    [
        "number_of_particles",
        "first_micrograph_name",
    ],
)


class AutoPick(JobType):
    def __eq__(self, other):
        if isinstance(other, AutoPick):  # check this
            return self._basepath == other._basepath
        return False

    def __hash__(self):
        return hash(("relion._parser.AutoPick", self._basepath))

    def __repr__(self):
        return f"AutoPick({repr(str(self._basepath))})"

    def __str__(self):
        return f"<AutoPick parser at {self._basepath}>"

    def _load_job_directory(self, jobdir):
        try:
            file = self._read_star_file(jobdir, "summary.star")
        except (RuntimeError, FileNotFoundError, ValueError):
            return []

        info_table = self._find_table_from_column_name("_rlnGroupNrParticles", file)
        if info_table is None:
            logger.debug(f"_rlnGroupNrParticles not found in file {file}")
            return []

        all_particles = self.parse_star_file("_rlnGroupNrParticles", file, info_table)
        num_particles = sum([int(n) for n in all_particles])

        first_mc_micrograph = self.parse_star_file(
            "_rlnMicrographName", file, info_table
        )[0]

        return [ParticlePickerInfo(num_particles, first_mc_micrograph)]

    @staticmethod
    def for_cache(partpickinfo):
        return str(partpickinfo.number_of_particles)



import logging
import pathlib

from relion._parser.autopick import ParticlePickerInfo
from relion._parser.jobtype import JobType

logger = logging.getLogger("relion._parser.cryolo")


class Cryolo(JobType):
    def __eq__(self, other):
        if isinstance(other, Cryolo):  # check this
            return self._basepath == other._basepath
        return False

    def __hash__(self):
        return hash(("relion._parser.Cryolo", self._basepath))

    def __repr__(self):
        return f"Cryolo({repr(str(self._basepath))})"

    def __str__(self):
        return f"<Cryolo parser at {self._basepath}>"

    @property
    def jobs(self):
        return sorted(
            d.resolve().name
            for d in self._basepath.iterdir()
            if d.is_symlink() and "crYOLO" in str(d)
        )

    def _load_job_directory(self, jobdir):
        num_particles = 0
        for star_file in (self._basepath / jobdir / "picked_stars").glob("*"):
            try:
                file = self._read_star_file(
                    jobdir, pathlib.Path("picked_stars") / star_file.name
                )
            except (RuntimeError, FileNotFoundError, ValueError):
                return []

            info_table = self._find_table_from_column_name("_rlnCoordinateX", file)
            if info_table is None:
                logger.debug(f"_rlnCoordinateX not found in file {file}")
                return []

            all_particles = self.parse_star_file("_rlnCoordinateX", file, info_table)
            num_particles += len(all_particles)

        # all of this just tracks back to a micrograph name from the MotionCorrection job
        try:
            jobfile = self._read_star_file(jobdir, "job.star")
        except (RuntimeError, FileNotFoundError, ValueError):
            return []
        info_table = self._find_table_from_column_name("_rlnJobOptionVariable", jobfile)
        if info_table is None:
            logger.debug(f"_rlnJobOptionVariable not found in file {jobfile}")
            return []
        variables = self.parse_star_file("_rlnJobOptionVariable", jobfile, info_table)
        inmicindex = variables.index("in_mic")
        ctffilename = pathlib.Path(
            self.parse_star_file("_rlnJobOptionValue", jobfile, info_table)[inmicindex]
        )
        ctffile = self._read_star_file_from_proj_dir(
            ctffilename.parts[0], ctffilename.relative_to(ctffilename.parts[0])
        )
        info_table = self._find_table_from_column_name("_rlnMicrographName", ctffile)
        if info_table is None:
            logger.debug(f"_rlnMicrographName not found in file {ctffile}")
            return []
        micrograph_name = self.parse_star_file(
            "_rlnMicrographName", ctffile, info_table
        )[0]

        return [ParticlePickerInfo(num_particles, micrograph_name)]

    @staticmethod
    def for_cache(partpickinfo):
        return str(partpickinfo.number_of_particles)
